#! /usr/bin/env python

"""Load function signatures from .sql file,
prepare them for listing in GRANT statements.
"""

import re, sys

rx = r"""
^
create [^(]+ function ( [^(]+ )
[(] ( [^)]* ) [)]
"""

rx_secdef = r"""security\s+definer"""


rc = re.compile(rx, re.I | re.M | re.X)
sc = re.compile(r"\s+")
rc_sec = re.compile(rx_secdef, re.I | re.X)

for fn in sys.argv[1:]:
    sql = open(fn).read()
    pos = 0
    while 1:
        m = rc.search(sql, pos)
        if not m:
            break
        pos = m.end()

        m2 = rc.search(sql, pos)
        if m2:
            xpos = m2.end()
        else:
            xpos = len(sql)
        secdef = False
        m2 = rc_sec.search(sql, pos, xpos)
        if m2:
            secdef = True

        fname = m.group(1).strip()
        fargs = m.group(2)

        alist = fargs.split(',')
        tlist = []
        for a in alist:
            toks = sc.split(a.strip().lower())
            if toks[0] == "out":
                continue
            if toks[0] in ("in", "inout"):
                toks = toks[1:]
            # just take last item
            tlist.append(toks[-1])

        ln = "    %s(%s)," % (fname, ", ".join(tlist))
        if secdef:
            ln = "%-72s -- SECDEF" % (ln)
        print ln

